import { URL } from "url";

import Mirai, { Message, MessageType } from "mirai-ts";

import { time } from "../../common/language";
import { redisStackBuilder } from "../../infra";

import { initFeedEvent } from "./feed-list";

type NotifitionConfig = Record<string | number, { atAll: boolean }>;
const redis = redisStackBuilder.mkChild("live-room-notifition");

export const install = async (mirai: Mirai) => {
  const feedEvent = await initFeedEvent(redis);

  mirai.on("GroupMessage", async (msg) => {
    if (!msg.isAt()) return;

    const isStart = msg.plain.includes("开播") && msg.plain.includes("通知");
    const isEnd = msg.plain.includes("下播") && msg.plain.includes("通知");
    const isChange = msg.plain.includes("修改") || msg.plain.includes("通知");
    const matchKeyWord = isStart || isEnd || isChange;

    const [, roomid] = msg.plain.match(/.*?(\d+).*?/) || [];

    if (!roomid && matchKeyWord) msg.reply("请提供一下直播间ID");

    if (!roomid || !matchKeyWord) return;

    const gId = msg.sender.group.id;
    const atAll =
      msg.plain.includes("全体成员") || msg.plain.includes("所有人");

    if (isStart)
      await redis
        .mkChild("live-start")
        .json<NotifitionConfig>(roomid)
        .modify((v) => ({ ...v, [gId]: { atAll } }), {});

    if (isEnd)
      await redis
        .mkChild("live-end")
        .json<NotifitionConfig>(roomid)
        .modify((v) => ({ ...v, [gId]: { atAll: false } }), {});

    if (isChange)
      await redis
        .mkChild("live-changed")
        .json<NotifitionConfig>(roomid)
        .modify((v) => ({ ...v, [gId]: { atAll: false } }), {});

    await msg.reply("好的，我记住了", true);
  });

  feedEvent.on("live-start", async (room) => {
    mirai.logger.info(`live-start: [${room.roomid}] ${room.title}`);
    const config = await redis
      .mkChild("live-start")
      .json<NotifitionConfig>(room.roomid)
      .get({});

    const link = new URL(room.link);
    link.search = "";

    const msg: MessageType.MessageChain = [
      Message.Plain(`${time.nyanpasu()}米娜桑～\n`),
      Message.Plain(`${room.uname}开播啦，今天的节目是：\n`),
      Message.Plain(`『${room.title}』`),
      Message.Image(undefined, room.pic),
      Message.Plain(link.toString()),
    ];

    mirai.logger.info(
      `live-start: [${room.roomid}] notify: ${Object.keys(config).join(",")}`
    );
    Object.entries(config).map(async ([groupId, { atAll }]) => {
      await mirai.api.sendGroupMessage(
        atAll ? [Message.AtAll(), ...msg] : msg,
        Number(groupId)
      );
    });
  });

  feedEvent.on("live-stop", async (room) => {
    mirai.logger.info(`live-stop: [${room.roomid}] ${room.title}`);
    const config = await redis
      .mkChild("live-stop")
      .json<NotifitionConfig>(room.roomid)
      .get({});

    mirai.logger.info(
      `live-stop: [${room.roomid}] notify: ${Object.keys(config).join(",")}`
    );
    Object.entries(config).map(async ([groupId]) => {
      await mirai.api.sendGroupMessage(
        `${room.uname}下播了，『${room.title}』告一段落，敬请期待下一场直播`,
        Number(groupId)
      );
    });
  });

  feedEvent.on("live-changed", async (prev, cur) => {
    mirai.logger.info(`live-changed: [${cur.roomid}] ${cur.title}`);

    const titleChanged = cur.title !== prev.title;
    const coverChanged = cur.pic !== prev.pic;

    if (!titleChanged && !coverChanged) return;

    const config = await redis
      .mkChild("live-changed")
      .json<NotifitionConfig>(cur.roomid)
      .get({});

    const msg: MessageType.MessageChain = [
      Message.Plain(`${cur.uname}修改了直播间`),
    ];

    if (titleChanged && !coverChanged)
      msg.push(Message.Plain(`标题：『${cur.uname}』`));
    if (!titleChanged && coverChanged)
      msg.push(Message.Plain(`封面：`), Message.Image(cur.pic));
    if (titleChanged && coverChanged)
      msg.push(Message.Plain(`标题和封面：`), Message.Image(cur.pic));

    mirai.logger.info(
      `live-changed: [${cur.roomid}] notify: ${Object.keys(config).join(",")}`
    );
    Object.entries(config).map(async ([groupId]) => {
      await mirai.api.sendGroupMessage(msg, Number(groupId));
    });
  });
};
